// Skin Smoother Filter for Friction2D by LinuxCreative
// Forked from:
// Bilateral Filter for Skin.
// URL: https://www.shadertoy.com/view/XtVGWG

#version 330 core
#define SIGMA 10.0
#define MSIZE 15

layout(location = 0) out vec4 fragColor;
layout(pixel_center_integer) in vec4 gl_FragCoord;

uniform sampler2D texture;

uniform vec2 resolution;

uniform float bSigma; //filter size
uniform float yVal; //blue diff B-Y
uniform float zVal; //red diff R-Y
uniform float spread;
uniform int check; //0
uniform int gammaCorrection; //0
float kernel[MSIZE];

float normpdf(in float x, in float sigma) {
	return 0.39894 * exp(-0.5 * x * x/ (sigma * sigma)) / sigma;
}

float normpdf3(in vec3 v, in float sigma) {
	return 0.39894 * exp(-0.5 * dot(v,v) / (sigma * sigma)) / sigma;
}

float normalizeColorChannel(in float value, in float min, in float max) {
    return (value - min)/(max-min);
}

void main(void) {
	vec3 c = texture2D(texture, (gl_FragCoord.xy / resolution.xy)).rgb;
    const int kSize = (MSIZE - 1) / 2;
    vec3 final_colour = vec3(0.0);
    float Z = 0.0;
    
	//create the 1-D kernel
	for (int j = 0; j <= kSize; ++j) {
		kernel[kSize+j] = kernel[kSize-j] = normpdf(float(j), SIGMA);
	}
    float bZ = 1.0 / normpdf(0.0, bSigma);    

    vec3 cc;
    float factor;
    //read out the texels
    for (int i=-kSize; i <= kSize; ++i) {
        for (int j=-kSize; j <= kSize; ++j) {
            cc = texture2D(texture, (gl_FragCoord.xy+vec2(float(i),float(j))) / resolution.xy).rgb;
            factor = normpdf3(cc-c, (bSigma / 10.0)) * bZ * kernel[kSize+j] * kernel[kSize+i];
            Z += factor;
            if (gammaCorrection > 0) {
            	final_colour += factor * pow(cc, vec3(2.2));
            } else {
            	final_colour += factor * cc;
            }
        }
    }
    
    if (gammaCorrection > 0) {
    	fragColor = vec4(pow(final_colour / Z, vec3(1.0/2.2)), 1.0);
    } else {
    	fragColor = vec4(final_colour / Z, 1.0);
    }
    
    // skin detection
    bool isSkin = false;
    float yUpper = yVal + spread;
    float zUpper = zVal + spread;
    vec4 rgb = fragColor * 255.0;
    vec4 ycbcr = rgb;
    ycbcr.x = 16.0 + rgb.x*0.257 + rgb.y*0.504 + rgb.z*0.098;
    ycbcr.y = 128.0 - rgb.x*0.148 - rgb.y*0.291 + rgb.z*0.439;
    ycbcr.z = 128.0 + rgb.x*0.439 - rgb.y*0.368 - rgb.z*0.071;
    if (ycbcr.y > yVal && ycbcr.y < yUpper &&
        ycbcr.z > zVal && ycbcr.z < zUpper) {
        isSkin = true; 
    }
    
    // passthrough for non-skin
    if (check > 0 || !isSkin) {
        fragColor = vec4(texture2D(texture, gl_FragCoord.xy / resolution.xy).xyz, 1.0); 
    }
    // show detected skin in blue
    if (check > 0 && isSkin) {
       	vec2 uv = gl_FragCoord.xy / resolution.xy;
    	vec4 color = texture2D(texture, uv);
        fragColor = vec4(color.x*0.3, color.y*0.7, color.z*1.0, 1.0);
    }
}
